package JournalEntry;

use strict;
use warnings;
use Carp;

use Foundation;
use Foundation::Functions;
use HTTP::Cookies;
use SOAP::Lite;
use Digest::MD5 'md5_hex';
use Date::Simple;

require Exporter;

our @ISA = qw(Exporter);

sub new {
  # Typical Perl constructor
  # See 'perltoot' for details
  my $proto = shift;
  my $class = ref($proto) || $proto;
  my $self = {
    'id'            => undef,
    'body'          => undef,
    'subject'       => undef,
    'comments'      => undef,
    'date'     	    => undef,
    'topic'    	    => 34,
    'type'          => undef,
    'host'          => undef,
    'creation_date' => undef
  };
  bless ($self, $class);

  # Save any parameters passed in
  my %params = @_;
  $self->{'id'} = $params{'id'};
  $self->{'host'} = $params{'host'};
  
  # Set the creation date
  $self->{'creation_date'} = scalar localtime;

  ## Set up the link to the journal website
  $self->connectSite();
    
  return $self;
}

sub uploadEntry {

  my $self = shift;
  my $journal = $self->{'journal'};
  my $new_id;

  # Upload the entry to the server
  eval{ $new_id = $journal->add_entry( subject  => $self->subject(),
                                       body     => $self->body(),
                                       discuss  => $self->comments(),
                                       tid      => $self->topic(),
                                       posttype => $self->type() )->result; };

  $self->{'id'} = $new_id if $new_id;
  return $new_id;
}

sub overwriteEntry {

  my $self = shift;
  my $journal = $self->{'journal'};
  my $id = $self->{'id'};
  my $rc;

  # Overwrite the existing entry on the server
  eval{ $rc = $journal->modify_entry( $id,
                                    subject  => $self->subject(),
                                    body     => $self->body(),
                                    tid      => $self->topic(),
                                    discuss  => $self->comments(),
                                    posttype => $self->type() )->result; };

  return $rc;
}

sub saveEntry {

  my $self = shift;
  my $userDefaults = NSUserDefaults->standardUserDefaults;
  my $offlineEntries = $userDefaults->objectForKey("writeSavedEntries");
  my $mutArray = NSMutableArray->arrayWithArray($offlineEntries);
    
  # Build up a dictionary to hold the entry
  my $entryHash = NSMutableDictionary->dictionary;
  $entryHash->setObject_forKey($self->body, "entry");
  $entryHash->setObject_forKey($self->subject, "subject");
  $entryHash->setObject_forKey($self->comments, "comments");
  $entryHash->setObject_forKey($self->type, "posttype");
  $entryHash->setObject_forKey($self->creation_date, "creation_date");

  # Add the entry to the array
  $mutArray->insertObject_atIndex($entryHash, 0);
  
  # Save the new array
  $userDefaults->setObject_forKey($mutArray, "writeSavedEntries");
  
  return 1;
}

# Get an entry from the server
sub retrieveEntry {

  my $self = shift;
  my $id   = $self->{'id'};
  my $journal = $self->{'journal'};
  my $entry;

  # Get the journal entry from the site  
  unless($entry = $journal->get_entry($id)->result) {
    return 0;
  }

  # Save the details of the entry into the journal
  $self->{'body'}     = $entry->{'body'};
  $self->{'subject'}  = $entry->{'subject'};
  $self->{'date'}     = $entry->{'date'};
  $self->{'topic'}    = $entry->{'topic'};
  $self->{'comments'} = $entry->{'discussion_url'};
  $self->{'type'}     = $entry->{'posttype'};
  $self->{'id'}       = $id;

  return 1;
}

# Delete an entry from the server
sub deleteEntry {

  my $self = shift;
  croak "No entry id defined" unless(defined $self->{'id'});
  my $id   = $self->{'id'};
  my $journal = $self->{'journal'};
  my $rc;

  # Upload the entry to the server
  eval{ $rc = $journal->delete_entry($id)->result; };

  return $rc;
}

#
# Public access procedures
#
sub body {
  my $self = shift;
  if(@_ == 0) {
    return $self->{'body'};
  }
  else {
    $self->{'body'} = shift;
  }
}
sub subject {
  my $self = shift;
  if(@_ == 0) {
    return $self->{'subject'};
  }
  else {
    $self->{'subject'} = shift;
  }
}
sub date {
  my $self = shift;
  if(@_ == 0) {
    return $self->{'date'};
  }
  else {
    $self->{'date'} = shift;
  }
}
sub topic {
  my $self = shift;
  if(@_ == 0) {
    return $self->{'topic'};
  }
  else {
    $self->{'topic'} = shift;
  }
}
sub comments {
  my $self = shift;
  if(@_ == 0) {
    return $self->{'comments'};
  }
  else {
    $self->{'comments'} = shift;
  }
}
sub type {
  my $self = shift;
  if(@_ == 0) {
    return $self->{'type'};
  }
  else {
    $self->{'type'} = shift;
  }
}
sub id {
  my $self = shift;
  if(@_ == 0) {
    return $self->{'id'};
  }
  else {
    $self->{'id'} = shift;
  }
}

sub creation_date {
  my $self = shift;
  if(@_ == 0) {
    return $self->{'creation_date'};
  }
  else {
    $self->{'creation_date'} = shift;
  }
}

sub formattedBody {
  my $self = shift;
  my $formattedBody = $self->{'body'};
  
  # If this was a 'plain-text' posting, add the line breaks
  # and paragraphs.
  if($self->{'type'} == 1) {
    $formattedBody =~ s/\n\n/<p>/g; # Maybe we should close as well?
    $formattedBody =~ s/\n/<br>/g;
  }
  
  return $formattedBody;
}

#
# Stolen from example by pudge at http://use.perl.org/~pudge/journal/3459
# 
sub bakeUserCookie {
  my($self, $uid, $passwd) = @_;
  my $cookie = $uid . '::' . md5_hex($passwd);
  $cookie =~ s/(.)/sprintf("%%%02x", ord($1))/ge;
  $cookie =~ s/%/%25/g;
  return $cookie;
}

#
# Set up a link to the journal, and save the
# resulting SOAP::Lite object for later use.
#
sub connectSite {
    my $self = shift;
    my $host = $self->{'host'};
    my $userDefaults = NSUserDefaults->standardUserDefaults;
    my $uid = $userDefaults->stringForKey($host."_UserID");
    my $password = $userDefaults->stringForKey($host."_Password");
    
    my $uri = "http://$host/Slash/Journal/SOAP";
    my $proxy = "http://$host/journal.pl";
    my $cookie = HTTP::Cookies->new;
    $cookie->set_cookie(0, user => $self->bakeUserCookie($uid, $password), '/', $host);
    
    my $journal = SOAP::Lite->uri($uri)
                            ->proxy($proxy, cookie_jar => $cookie);
    $self->{'journal'} = $journal;
}

1;

#
# JournalEntry.pm
# Copyright 2002 Garth Douglass
# This program is distributed under the terms of the GNU General Public License, see COPYING file for details
#
